# Naslov        : Rkurs3.R  ;
# Cilj          : Analiza, vizualizacija, interpolacija prostornih podataka;
# Keator        : Milan Kilibarda, GRF kili@grf.bg.ac.rs  ;
# last update   : Jun 2011.  ;
# inputs        :  Podaci.shp , Krupanj.shp, DEM.asc, NAGIB.asc  ...
# outputs       :   ...

rm(list=ls())
library(sp)
library(rgdal)
library(gstat)
library(plotGoogleMaps)

setwd("J:/geostat knjige/R_kurs_jun2011/4/KrupanjSCRIPT" )

### Importovanje SHP podataka u R #############################################$
sliv<-readOGR(".","Krupanj")
str(sliv)

podaci<-readOGR(".","Podaci")
str(podaci)
podaci@proj4string
## Definisanje DKS sa Potpunijim parametrima
proj4string(podaci) <- CRS("+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y_0=0 +ellps=bessel +towgs84=574.027,170.175,401.545,4.88786,-0.66524,-13.24673,0.99999311067 +units=m")
proj4string(sliv) <- CRS("+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y_0=0 +ellps=bessel +towgs84=574.027,170.175,401.545,4.88786,-0.66524,-13.24673,0.99999311067 +units=m")
podaci@proj4string
###  Pregled atributnih podataka i njihove klase ###############################
str(podaci)
str(podaci@data)

names(podaci@data)

names(podaci@data) <- c ("RB","mesto", "x","y","kultura","pH_H2O" ,"pH_KCl", "humus","ukupni","prist","prist1")
podaci$humus
podaci@data[,"humus"]



###   Podaci  na Google Earth-u  ###############################################

swgs <- spTransform(sliv, CRS("+proj=longlat +datum=WGS84"))
writeOGR(obj=swgs, dsn="swgs.kml", layer="swgs", driver="KML")



podaciwgs <- spTransform(podaci, CRS("+proj=longlat +datum=WGS84"))
writeOGR(obj=podaciwgs, dsn="podaci.kml", layer="podaci", driver="KML")


### Podaci na Google Maps ####################################
? plotGoogleMaps

N<- plotGoogleMaps(sliv,filename='slivKrupanj.htm')

# http://colorbrewer2.org/
library(RColorBrewer)


S<- plotGoogleMaps(sliv,filename='sliv.htm',  colPalette = "#000000",  strokeOpacity = 0.3)
P<- plotGoogleMaps(podaci,filename='podaci.htm',zcol='humus')

# Sve zajedno
S2<- plotGoogleMaps(sliv,add=T,  ,colPalette = "#ADDD8E", mapTypeId='TERRAIN', strokeOpacity = 0.3)
P2<- plotGoogleMaps(podaci, previousMap=S2,filename='sve.htm',zcol='humus')


? bubbleGoogleMaps

m<-bubbleGoogleMaps(podaci,zcol="humus",  max.radius = 400,filename='hum1.htm')
m<-bubbleGoogleMaps(podaci,zcol="humus",  max.radius = 400, do.sqrt=F,filename='hum2.htm')


# Sve zajedno
boje<-colorRampPalette(brewer.pal(5, "Reds"))(9)[4:8]
m<-bubbleGoogleMaps(podaci,
                  previousMap=S2,
                  zcol="humus",
                  max.radius = 400,
                  do.sqrt=F,
                  filename='sveHimus.htm',
                  colPalette =boje,
                  mapTypeId='TERRAIN',
                  strokeColor="#FF0000" )
                  
m<-bubbleGoogleMaps(podaci,
                  previousMap=S2,
                  zcol="pH_H2O",
                  max.radius = 400,
                  do.sqrt=T,
                  filename='svepH_H20.htm',
                  colPalette =boje,
                  mapTypeId='TERRAIN',
                  strokeColor="#FF0000" )

### Bubble plot humusa   ###################################################
humus.plt <- bubble(podaci, "humus", col="Red", maxsize=2)
print(humus.plt)




######  Histogrami svih podataka ###############################################
imena<-  names(podaci@data)
imena
imena<-imena[-1:-5]
imena

summary(podaci)
summary(podaci$humus)
var(podaci$humus)
hist(podaci$humus,25)

hist(podaci@data[,imena[2]],25)

# sirovi podaci

par(mfrow=c(2, 3), mar=c(4.5,4.5,0,1))
for(hmc in imena){
hist(podaci@data[,hmc], col="grey", breaks=25, main="", xlab=hmc, ylab="")
}
dev.off()

## QQ plotovi
par(mfrow=c(2, 3))
for(hmc in imena){
qqnorm( podaci@data[,hmc], main=paste("Normal QQ plot",hmc) )
qqline(podaci@data[,hmc],lty=2)
}
dev.off()

## Test normalosti
shapiro.test(podaci@data[,'humus'])
lapply(imena, function(x) shapiro.test(podaci@data[,x]) )

# log transformacija
hist(log1p(podaci$humus),25)

par(mfrow=c(2, 3), mar=c(4.5,4.5,0,1))
for(hmc in imena){
hist(log1p(podaci@data[,hmc]), col="grey", breaks=25, main="", xlab=paste("log1p(",hmc,")",sep=""), ylab="")
}

## QQ plotovi
par(mfrow=c(2, 3))
for(hmc in imena){
qqnorm( log1p(podaci@data[,hmc] ), main=paste("Normal QQ plot log1p ",hmc) )
qqline(log1p(podaci@data[,hmc] ),lty=2)
}

dev.off()
## Test normalosti
lapply(imena, function(x) shapiro.test(log1p( podaci@data[,x]) ) )

# Asimetricnost
library(moments)
 skewness(podaci@data[,imena] )
 skewness(log1p(podaci@data[,imena]) )


####  Korelacija merenja                 ####################################
cor(log1p(podaci@data[,imena]), use="complete.obs")
cor(log1p(podaci@data[,imena]), use="complete.obs")[imena,"humus"]

#######  Pomocni prediktori (gridovi) ##############################################

krupanjgrid <- readGDAL("DEM.asc")
names(krupanjgrid ) <- "DEM"
str(krupanjgrid)

krupanjgrid$NAGIB <- readGDAL("NAGIB.asc")$band1
str(krupanjgrid)
proj4string(krupanjgrid) <- CRS("+proj=tmerc +lat_0=0 +lon_0=21 +k=0.9999 +x_0=7500000 +y_0=0 +ellps=bessel +towgs84=574.027,170.175,401.545,4.88786,-0.66524,-13.24673,0.99999311067 +units=m")
#### Preklop tacak i pomocnih prediktora #######################################
tacke<-overlay(krupanjgrid, podaci)

str(tacke)
podaci@data<- cbind(podaci@data,tacke@data)

str(podaci)
####  Korelacija merenja                 ####################################
cor(log1p(podaci@data[,c(imena,"DEM","NAGIB")]), use="complete.obs")


#####  IDW predikcija  HUMUSA i pH_H20 ######################################################
summary(podaci$humus)
summary(podaci$pH_H2O)

# koristimo idw funkciju
idw.Humus <- idw(humus~1, podaci, krupanjgrid, idp = 2.5)
idw.pH_H2O <- idw(pH_H2O~1, podaci, krupanjgrid, idp = 2.5)

str(  idw.Humus )
summary(idw.Humus$var1.pred)
spplot(idw.Humus,"var1.pred", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="IDW Humus")
spplot(idw.pH_H2O,"var1.pred" , col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)),main="IDW pH_H2O")



par(mar=c(2,2,2,2))
image(idw.Humus["var1.pred"], col=terrain.colors(20))
contour(idw.Humus["var1.pred"], add=TRUE, drawlabels=FALSE, col='brown')
points(podaci, pch=4, cex=0.5)
title('IDW humus')

# koristimo gstat funkciju    za Humus
Humus_idw = gstat(id = "humus", formula = humus~1, data = podaci, set =list(idp = 2.5))
idw.H<- predict.gstat(Humus_idw, krupanjgrid)

str(idw.H)
summary(idw.H$humus.pred)

out = gstat.cv(Humus_idw, nfold=5 )   # kros validacija

str(out)

sqrt(mean(out$residual^2))
bubble(out,"residual")

# koristimo gstat funkciju   pH_H2O
pH_H2O_idw = gstat(id = "pH_H2O", formula = pH_H2O~1, data = podaci, set =list(idp = 2.5))
idw.pH_H2O<- predict.gstat(pH_H2O_idw, krupanjgrid)


str(idw.pH_H2O)
summary(idw.pH_H2O$pH_H2O.pred)

out = gstat.cv(pH_H2O_idw , nfold=5 )

str(out)
idw.ph.plt<-spplot(idw.pH_H2O,"pH_H2O.pred", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="IDW pH_H2O")
sqrt(mean(out$residual^2))    # 0.8811697
idwr_plt<-bubble(out,"residual",main=" IDW reziduali")
print(idwr_plt)

# Rezultati u ASC fajlu
writeGDAL(idw.H["humus.pred"],"idw_humus.asc", "AAIGrid")
writeGDAL(idw.pH_H2O["pH_H2O.pred"],"idw_pH_H2O.asc", "AAIGrid")
### Izbor najboljeg tezinskog koeficijenta za IDW primer za HUMUS

# funkcija koja ce raditi KV
do_cv = function(idp) {
humus_idw = gstat(id = "humus", formula = humus~1, data = podaci, set =list(idp = idp))
  out = gstat.cv(humus_idw,nfold=5)
  return(sqrt(mean(out$residual^2))) # RMSE kao optimalni kriterijum
}

idw_pow = seq(0.1,5, by = 0.2) # tezinski koef. koji ce biti razmatrani
cv_vals = sapply(idw_pow, do_cv) # RMSE za sve koef.

print(data.frame(idp = idw_pow, cv_rmse = cv_vals))     # Lista rezultata

### Ordinary Kriging #############################################################

# Variogram
# fitovanje variograma :

humus.svar <- variogram(log1p(pH_H2O)~1, podaci)
plot(humus.svar)
humus.svar

# initial variogram:
humus.ivgm <- vgm(nugget=0, model="Lin", range=sqrt(diff(podaci@bbox[1,])^2 + diff(podaci@bbox[2,])^2)/4, psill=var(log1p(podaci$pH_H2O)) )
humus.vgm <- fit.variogram(humus.svar, model=humus.ivgm)
humus.vgm
humus.vgm.plt <- plot(humus.svar, humus.vgm, pch="+", pl=TRUE,  col="black", main="log1p(pH_H2O)")
print(humus.vgm.plt)

kr.out <- gstat(id="pH_H2O",formula =log1p(pH_H2O)~1,data = podaci,model=humus.vgm)

kr.pH_H2O<- predict.gstat(kr.out, krupanjgrid)     # nekoliko minuta

kr.pH_H2O$pH_H2O<-expm1(kr.pH_H2O$pH_H2O.pred )

spplot(kr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="OK  pH_H2O")

kr.pH_H2O.plt<- spplot(kr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="OK  pH_H2O")
print(idw.ph.plt, split=c(1,1,2,1), more=T)
print(kr.pH_H2O.plt, split=c(2,1,2,1), more=F)


dev.off()

out = gstat.cv(kr.out, nfold=5 )   # kros validacija

str(out)
reziduali<- expm1(out$observed)- expm1(out$pH_H2O.pred)
sqrt(mean(reziduali^2))     # 0.83    za IDW 0.88
out$reziduali<-reziduali
bubble(out,"reziduali")
okr_plt<-bubble(out,"residual",main=" OK reziduali")
print(okr_plt)

# Rezultati u ASC fajlu
writeGDAL(kr.pH_H2O["pH_H2O"],"kr_pH_H2O.asc", "AAIGrid")

### Univerzalni Kriging ########################################################


pH_H2O.svar <- variogram(log1p(pH_H2O)~x+y, podaci)
plot(pH_H2O.svar)
pH_H2O.svar
# initial variogram:
pH_H2O.ivgm <- vgm(nugget=0, model="Exp", range=sqrt(diff(podaci@bbox[1,])^2 + diff(podaci@bbox[2,])^2)/4, psill=var(log1p(podaci$pH_H2O)) )
pH_H2O.vgm <- fit.variogram(pH_H2O.svar, model=pH_H2O.ivgm)
pH_H2O.vgm
pH_H2O.vgm.plt <- plot(pH_H2O.svar, pH_H2O.vgm, pch="+", pl=TRUE,  col="black", main="logt(pH_H2O)")
print(pH_H2O.vgm.plt)

ukr.out <- gstat(id="pH_H2O",formula =log1p(pH_H2O)~x+y,data = podaci,model=pH_H2O.vgm)

ukr.pH_H2O<- predict.gstat(ukr.out, krupanjgrid)   # nekoliko min

ukr.pH_H2O$pH_H2O<-expm1(ukr.pH_H2O$pH_H2O.pred)

spplot(ukr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="UK pH_H2O")

ukr.pH_H2O.plt<- spplot(ukr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="UK pH_H2O")
print(idw.ph.plt, split=c(1,1,3,1), more=T)
print(kr.pH_H2O.plt, split=c(2,1,3,1), more=T)
print(ukr.pH_H2O.plt, split=c(3,1,3,1), more=F)

dev.off()

out = gstat.cv(ukr.out, nfold=5 )   # kros validacija

str(out)
reziduali<- expm1(out$observed)- expm1(out$pH_H2O.pred)
sqrt(mean(reziduali^2))   # 0.816 za UK ,  0.82 za OK    , 0.88  za IDW
out$reziduali<-reziduali
ukr_plt<-bubble(out,"residual",main=" UK reziduali")
print(ukr_plt)

# Rezultati u ASC fajlu
writeGDAL(ukr.pH_H2O["pH_H2O"],"ukr_pH_H2O.asc", "AAIGrid")

### Regresioni Kriging ########################################################


pH_H2O.svar <- variogram(log1p(pH_H2O)~DEM, podaci)
plot(pH_H2O.svar)
pH_H2O.svar
# initial variogram:
pH_H2O.ivgm <- vgm(nugget=0, model="Exp", range=sqrt(diff(podaci@bbox[1,])^2 + diff(podaci@bbox[2,])^2)/4, psill=var(log1p(podaci$pH_H2O)) )
pH_H2O.vgm <- fit.variogram(pH_H2O.svar, model=pH_H2O.ivgm)
pH_H2O.vgm
pH_H2O.vgm.plt <- plot(pH_H2O.svar, pH_H2O.vgm, pch="+", pl=TRUE,  col="black", main="logt(pH_H2O)")
print(pH_H2O.vgm.plt)

rkr.out <- gstat(id="pH_H2O",formula =log1p(pH_H2O)~DEM,data = podaci,model=pH_H2O.vgm)

rkr.pH_H2O<- predict.gstat(rkr.out, krupanjgrid)   # nekoliko min

rkr.pH_H2O$pH_H2O<-expm1(rkr.pH_H2O$pH_H2O.pred)

spplot(rkr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="RK pH_H2O")

rkr.pH_H2O.plt<- spplot(rkr.pH_H2O,"pH_H2O", col.regions=grey(c(rev(seq(0,0.9,0.05)), 0)) ,main="RK pH_H2O")
print(idw.ph.plt, split=c(1,1,4,1), more=T)
print(kr.pH_H2O.plt, split=c(2,1,4,1), more=T)
print(ukr.pH_H2O.plt, split=c(3,1,4,1), more=T)
print(rkr.pH_H2O.plt, split=c(4,1,4,1), more=F)
dev.off()

out = gstat.cv(rkr.out, nfold=5 )   # kros validacija

str(out)
reziduali<- expm1(out$observed)- expm1(out$pH_H2O.pred)
sqrt(mean(reziduali^2))   # 0.816 RK, 0.816 za UK ,  0.82 za OK    , 0.88  za IDW
out$reziduali<-reziduali
rkr_plt<-bubble(out,"residual",main=" RK reziduali")
print(rkr_plt)

print(idwr_plt, split=c(1,1,4,1), more=T)
print(okr_plt, split=c(2,1,4,1), more=T)
print(ukr_plt, split=c(3,1,4,1), more=T)
print(rkr_plt, split=c(4,1,4,1), more=F)

dev.off()

# Rezultati u ASC fajlu
writeGDAL(rkr.pH_H2O["pH_H2O"],"rkr_pH_H2O.asc", "AAIGrid")